#Make EigenImages from AT&T Cambridge Faces Database
#Author: Matt Shotwell
#Date: 9/7/10

#Function to read pgm files
pgm <- function(file) {
    if(!is.character(file) || !file.exists(file))
        stop("invalid file argument")
    con <- file(file, "rb")
    # Check for magic number (ascii "P5")
    if(scan(con, character(0), 1, quiet=TRUE)!="P5")
        stop("file not PGM")
    # Read width, height, maxval
    width  <- scan(con, integer(0), 1, quiet=TRUE)
    height <- scan(con, integer(0), 1, quiet=TRUE)
    maxval <- scan(con, integer(0), 1, quiet=TRUE)
    # Read data
    size <- ifelse(maxval > 255, 2, 1)
    data <- readBin(con, "integer", height*width, size, FALSE)
    close(con)
    return(matrix(data, height, width, byrow=TRUE))
}

#Load AT&T Faces Database
pix <- vector()
grp <- vector()
cnt <- 1
Nsub <- 40
Nimg <- 10
N <- Nsub * Nimg
for( sub in 1:Nsub ) {
    for( img in 1:Nimg ) {
        fname = paste("faces/s",sub,img,".pgm",sep="")
        cat(paste("loading", fname, "\n"))
        dat <- pgm(fname)
        pix <- c(pix, c(dat))
        grp <- c(grp, rep(cnt, prod(dim(dat))))
        cnt <- cnt + 1
    }
}
rm(fname, dat, cnt, sub, img)


#Stack data, perform pca, select top K principal components
cat("packaging pixel data for PCA\n")
pixmat <- unstack(pix, pix ~ grp)
cat("performing PCA\n")
pca <- prcomp(t(pixmat))

pcaPlot <- function() {
  h <- 112
  w <- 92
  pcasub <- as.matrix( pca$rotation[ ,1:20 ] )
  par(mfrow = c(4, 5), mai = c(0,0,0,0), xaxt="n", yaxt="n")
  for( i in 1:20 ) {
    stk <- data.frame(pca$rotation[,i], rep(1:w, rep(h,w)))
    uns <- as.matrix(unstack(stk))[h:1,]
    image( t(uns), col=gray(seq(0,1,length.out=1024))  )
  }
}
